# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {
    "convert_gymir_result_to_propulsion_power_series": "00_machinery_calculation.ipynb",
    "MachineryCalculation": "00_machinery_calculation.ipynb",
    "PmsLoadTable": "01_pms_basic.ipynb",
    "get_rated_power_from_power_source": "01_pms_basic.ipynb",
    "get_min_load_table_dict_from_proto_system": "01_pms_basic.ipynb",
    "get_min_load_table_dict_from_feems_system": "01_pms_basic.ipynb",
    "min_load_table_dict": "01_pms_basic.ipynb",
    "PmsLoadTableSimulationInterface": "01_pms_basic.ipynb",
    "OnPattern": "01_pms_basic.ipynb",
    "Load2OnPattern": "01_pms_basic.ipynb",
}

modules = ["machinery_calculation.py", "pms_basic.py"]

doc_url = "https://kevinkoosup.yum@sintef.no.github.io/RunFeemsSim/"

git_url = "https://"


def custom_doc_links(name):
    return None
