# Autogenerated by nbdev

d = {
    "settings": {
        "branch": "master",
        "doc_baseurl": "/RunFeemsSim/",
        "doc_host": "https://kevinkoosup.yum@sintef.no.github.io",
        "git_url": "https://github.com/SINTEF/FEEMS",
        "lib_path": "RunFeemsSim",
    },
    "syms": {
        "RunFeemsSim.machinery_calculation": {
            "RunFeemsSim.machinery_calculation.MachineryCalculation": (
                "machinery_calculation.html#machinerycalculation",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.__init__": (
                "machinery_calculation.html#machinerycalculation.__init__",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation._run_simulation": (
                "machinery_calculation.html#machinerycalculation._run_simulation",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation._set_equal_load_sharing_on_power_sources": (
                "machinery_calculation.html#machinerycalculation._set_equal_load_sharing_on_power_sources",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation._set_input_load_from_gymir_result": (
                "machinery_calculation.html#machinerycalculation._set_input_load_from_gymir_result",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation._set_input_load_time_interval_from_propulsion_power_time_series": (
                "machinery_calculation.html#machinerycalculation._set_input_load_time_interval_from_propulsion_power_time_series",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation._set_status_for_mechanical_system": (
                "machinery_calculation.html#machinerycalculation._set_status_for_mechanical_system",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.calculate_machinery_system_output_from_gymir_result": (
                "machinery_calculation.html#machinerycalculation.calculate_machinery_system_output_from_gymir_result",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.calculate_machinery_system_output_from_propulsion_power_time_series": (
                "machinery_calculation.html#machinerycalculation.calculate_machinery_system_output_from_propulsion_power_time_series",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.calculate_machinery_system_output_from_statistics": (
                "machinery_calculation.html#machinerycalculation.calculate_machinery_system_output_from_statistics",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.calculate_machinery_system_output_from_time_series_result": (
                "machinery_calculation.html#machinerycalculation.calculate_machinery_system_output_from_time_series_result",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.electric_system": (
                "machinery_calculation.html#machinerycalculation.electric_system",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.MachineryCalculation.system_is_not_electric": (
                "machinery_calculation.html#machinerycalculation.system_is_not_electric",
                "RunFeemsSim/machinery_calculation.py",
            ),
            "RunFeemsSim.machinery_calculation.convert_gymir_result_to_propulsion_power_series": (
                "machinery_calculation.html#convert_gymir_result_to_propulsion_power_series",
                "RunFeemsSim/machinery_calculation.py",
            ),
        },
        "RunFeemsSim.pms_basic": {
            "RunFeemsSim.pms_basic.PmsLoadTable": (
                "pms_basic.html#pmsloadtable",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.PmsLoadTable.__post_init__": (
                "pms_basic.html#pmsloadtable.__post_init__",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.PmsLoadTable.on_pattern": (
                "pms_basic.html#pmsloadtable.on_pattern",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.PmsLoadTable.sorted_load_table": (
                "pms_basic.html#pmsloadtable.sorted_load_table",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.PmsLoadTableSimulationInterface": (
                "pms_basic.html#pmsloadtablesimulationinterface",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.PmsLoadTableSimulationInterface.__init__": (
                "pms_basic.html#pmsloadtablesimulationinterface.__init__",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.PmsLoadTableSimulationInterface.set_status": (
                "pms_basic.html#pmsloadtablesimulationinterface.set_status",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.get_min_load_table_dict_from_feems_system": (
                "pms_basic.html#get_min_load_table_dict_from_feems_system",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.get_min_load_table_dict_from_proto_system": (
                "pms_basic.html#get_min_load_table_dict_from_proto_system",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.get_rated_power_from_power_source": (
                "pms_basic.html#get_rated_power_from_power_source",
                "RunFeemsSim/pms_basic.py",
            ),
            "RunFeemsSim.pms_basic.min_load_table_dict": (
                "pms_basic.html#min_load_table_dict",
                "RunFeemsSim/pms_basic.py",
            ),
        },
    },
}
